"""High level command line interface to hitch."""
from hitchkey import utils
from os import path, makedirs
import random
import string
import sys
import os


def new_hitch_dir():
    return path.join(
        path.expanduser("~/.hitch"),
        ''.join([
            random.choice(
                string.digits + string.lowercase
            ) for _ in range(6)
        ])
    )


def run():
    checkdirectory = os.getcwd()
    directories_checked = []
    keypy_filename = None

    python3, virtualenv = utils.check_python_and_virtualenv(
        None,
        None,
    )

    while not os.path.ismount(checkdirectory):
        directories_checked.append(checkdirectory)
        if os.path.exists("{0}{1}key.py".format(checkdirectory, os.sep)):
            keypy_filename = "{0}{1}key.py".format(checkdirectory, os.sep)
            break
        elif os.path.exists("{0}{1}hitch{1}key.py".format(checkdirectory, os.sep)):
            keypy_filename = "{0}{1}hitch{1}key.py".format(checkdirectory, os.sep)
            break
        else:
            checkdirectory = os.path.abspath(os.path.join(checkdirectory, os.pardir))

    if not keypy_filename:
        sys.stderr.write("key.py not found in the following directories:\n\n")
        sys.stderr.write('\n'.join(directories_checked))
        sys.stderr.write("\n\nSee http://hitchkey.readthedocs.org/en/latest/quickstart.html\n")
        sys.exit(1)

    keypy_directory = os.path.dirname(keypy_filename)

    gensymlink = os.path.abspath("gen")

    if os.path.exists(gensymlink):
        genpath = os.path.realpath(gensymlink)
    else:
        genpath = new_hitch_dir()

    if not path.exists(genpath):
        makedirs(genpath)
        utils.check_call([
            virtualenv,
            path.join(genpath, "hvenv"),
            "--no-site-packages",
            "-p", python3
        ])
        with open(path.join(genpath, "hvenv", "linkfile"), 'w') as handle:
            handle.write(keypy_directory)
        utils.check_call([path.join(genpath, "hvenv", "bin", "pip"), "install", "hitchrun"])
        os.symlink(genpath, path.join(keypy_directory, "gen"))

    hitchrun = path.abspath(path.join(genpath, "hvenv", "bin", "hitchrun"))
    os.execvp(hitchrun, [hitchrun] + sys.argv[1:])


if __name__ == '__main__':
    run()



from sys import stderr, exit
from subprocess import call, PIPE, Popen, STDOUT
from hitchkey import languagestrings
from functools import reduce
from os import path, getcwd


class CalledProcessError(Exception):
    """Re-implemented CalledProcessError, since it is not available < python 2.7."""
    pass


def check_output(command, stdout=PIPE, stderr=PIPE):
    """Re-implemented subprocess.check_output since it is not available < python 2.7."""
    return Popen(command, stdout=stdout, stderr=stderr).communicate()[0]


def check_call(command, shell=False):
    """Re-implemented subprocess.check_call since it is not available < python 2.7."""
    process = Popen(command, shell=shell)
    process.communicate()
    if process.returncode != 0:
        raise CalledProcessError
    return


def stop_everything(sig, frame):
    """Exit hitch."""
    exit(1)


def check_python_and_virtualenv(python, virtualenv):
    """Check that virtualenv and python3 are available and usable."""
    if python is None:
        if call(["which", "python3"], stdout=PIPE, stderr=PIPE) != 0:
            stderr.write(languagestrings.YOU_MUST_HAVE_PYTHON3_INSTALLED)
            stderr.flush()
            exit(1)
        python3 = check_output(["which", "python3"]).decode('utf8').replace("\n", "")
    else:
        if path.exists(python):
            python3 = python
        else:
            stderr.write("{0} not found.\n\n".format(python))
            stderr.write(languagestrings.YOU_MUST_HAVE_PYTHON3_INSTALLED)
            exit(1)

    python_version = check_output([python3, "-V"], stderr=STDOUT).decode('utf8')
    replacements = ('Python ', ''), ('\n', '')
    str_version = reduce(lambda a, kv: a.replace(*kv), replacements, python_version)
    tuple_version = tuple([int(x) for x in str_version.split('.')[:2]])

    if tuple_version < (3, 3):
        stderr.write(languagestrings.YOU_MUST_HAVE_VERSION_ABOVE_PYTHON33)
        exit(1)

    if virtualenv is None:
        if call(["which", "virtualenv"], stdout=PIPE, stderr=PIPE) != 0:
            stderr.write(languagestrings.YOU_MUST_HAVE_VIRTUALENV_INSTALLED)
            stderr.flush()
            exit(1)
        virtualenv = check_output(["which", "virtualenv"]).decode('utf8').replace("\n", "")
    else:
        if path.exists(virtualenv):
            if python is None:
                python = path.join(path.dirname(virtualenv), "python")
        else:
            stderr.write("{0} not found.\n".format(virtualenv))
            exit(1)

    return python3, virtualenv


def fail_if_spaces_in_path(filepath):
    """Raise error if spaces are found in a path."""
    if " " in filepath:
        stderr.write(languagestrings.SPACES_NOT_ALLOWED.format(getcwd()))
        exit(1)
    return filepath


def read_config(filename):
    """
    Read a standard ini file without section headers.
    """
    properties = {}
    if path.exists(filename):
        with open(filename, 'r') as handle:
            for line in "".join(handle.readlines()).split("\n"):
                if line != "" and not line.startswith(";"):
                    if line.count("=") != 1:
                        stderr.write("Error reading {0}\n".format(filename))
                        stderr.write("Line '{0}' does not follow key=val\n".format(line))
                        exit(1)
                    key, value = line.split("=")
                    properties[key] = value
    return properties
